@@ -12,8 +12,6 @@ GITHUB_API_AUTHORITY = "api.github.com"
 
 SUPPORTED_CHAIN_IDS = {
     1: "mainnet",
-    3: "ropsten",
-    4: "rinkeby",
     5: "goerli",
     11155111: "sepolia",
 }
