@@ -2,6 +2,8 @@ import functools
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
+    Dict,
     Optional,
     Sequence,
     Tuple,
@@ -13,19 +15,25 @@ from typing import (
 from eth_abi.codec import (
     ABICodec,
 )
+from eth_abi.registry import (
+    registry as default_registry,
+)
 from eth_typing import (
     ChecksumAddress,
     HexStr,
+    TypeStr,
 )
 from eth_utils import (
     add_0x_prefix,
     encode_hex,
     function_abi_to_4byte_selector,
+    is_binary_address,
+    is_checksum_address,
+    is_list_like,
     is_text,
 )
 from eth_utils.toolz import (
     pipe,
-    valmap,
 )
 from hexbytes import (
     HexBytes,
@@ -45,6 +53,10 @@ from web3._utils.abi import (
     get_receive_func_abi,
     map_abi_data,
     merge_args_and_kwargs,
+    named_tree,
+)
+from web3._utils.blocks import (
+    is_hex_encoded_block_hash,
 )
 from web3._utils.encoding import (
     to_hex,
@@ -63,35 +75,61 @@ from web3._utils.normalizers import (
     abi_string_to_text,
 )
 from web3.exceptions import (
-    ValidationError,
+    BlockNumberOutofRange,
+    Web3ValidationError,
 )
 from web3.types import (
     ABI,
     ABIEvent,
     ABIFunction,
+    BlockIdentifier,
+    BlockNumber,
     TxParams,
 )
 
 if TYPE_CHECKING:
-    from web3 import Web3  # noqa: F401
+    from web3 import (  # noqa: F401
+        AsyncWeb3,
+        Web3,
+    )
+
+
+def extract_argument_types(*args: Sequence[Any]) -> str:
+    """
+    Takes a list of arguments and returns a string representation of the argument types,
+    appropriately collapsing `tuple` types into the respective nested types.
+    """
+    collapsed_args = []
+
+    for arg in args:
+        if is_list_like(arg):
+            collapsed_nested = []
+            for nested in arg:
+                if is_list_like(nested):
+                    collapsed_nested.append(f"({extract_argument_types(nested)})")
+                else:
+                    collapsed_nested.append(_get_argument_readable_type(nested))
+            collapsed_args.append(",".join(collapsed_nested))
+        else:
+            collapsed_args.append(_get_argument_readable_type(arg))
+
+    return ",".join(collapsed_args)
 
 
 def find_matching_event_abi(
-    abi: ABI, event_name: Optional[str] = None,
-    argument_names: Optional[Sequence[str]] = None
+    abi: ABI,
+    event_name: Optional[str] = None,
+    argument_names: Optional[Sequence[str]] = None,
 ) -> ABIEvent:
-
     filters = [
-        functools.partial(filter_by_type, 'event'),
+        functools.partial(filter_by_type, "event"),
     ]
 
     if event_name is not None:
         filters.append(functools.partial(filter_by_name, event_name))
 
     if argument_names is not None:
-        filters.append(
-            functools.partial(filter_by_argument_name, argument_names)
-        )
+        filters.append(functools.partial(filter_by_argument_name, argument_names))
 
     event_abi_candidates = pipe(abi, *filters)
 
@@ -133,68 +171,74 @@ def find_matching_fn_abi(
         return function_candidates[0]
     else:
         matching_identifiers = name_filter(abi)
-        matching_function_signatures = [abi_to_signature(func) for func in matching_identifiers]
+        matching_function_signatures = [
+            abi_to_signature(func) for func in matching_identifiers
+        ]
 
         arg_count_matches = len(arg_count_filter(matching_identifiers))
         encoding_matches = len(encoding_filter(matching_identifiers))
 
         if arg_count_matches == 0:
-            diagnosis = "\nFunction invocation failed due to improper number of arguments."
+            diagnosis = (
+                "\nFunction invocation failed due to improper number of arguments."
+            )
         elif encoding_matches == 0:
-            diagnosis = "\nFunction invocation failed due to no matching argument types."
+            diagnosis = (
+                "\nFunction invocation failed due to no matching argument types."
+            )
         elif encoding_matches > 1:
             diagnosis = (
                 "\nAmbiguous argument encoding. "
-                "Provided arguments can be encoded to multiple functions matching this call."
+                "Provided arguments can be encoded to multiple functions "
+                "matching this call."
             )
 
+        collapsed_args = extract_argument_types(args)
+        collapsed_kwargs = dict(
+            {(k, extract_argument_types([v])) for k, v in kwargs.items()}
+        )
         message = (
-            "\nCould not identify the intended function with name `{name}`, "
-            "positional argument(s) of type `{arg_types}` and "
-            "keyword argument(s) of type `{kwarg_types}`."
-            "\nFound {num_candidates} function(s) with the name `{name}`: {candidates}"
-            "{diagnosis}"
-        ).format(
-            name=fn_identifier,
-            arg_types=tuple(map(type, args)),
-            kwarg_types=valmap(type, kwargs),
-            num_candidates=len(matching_identifiers),
-            candidates=matching_function_signatures,
-            diagnosis=diagnosis,
+            f"\nCould not identify the intended function with name `{fn_identifier}`, "
+            f"positional arguments with type(s) `{collapsed_args}` and "
+            f"keyword arguments with type(s) `{collapsed_kwargs}`."
+            f"\nFound {len(matching_identifiers)} function(s) with "
+            f"the name `{fn_identifier}`: {matching_function_signatures}{diagnosis}"
         )
 
-        raise ValidationError(message)
+        raise Web3ValidationError(message)
 
 
 def encode_abi(
-    web3: "Web3", abi: ABIFunction, arguments: Sequence[Any], data: Optional[HexStr] = None
+    w3: Union["AsyncWeb3", "Web3"],
+    abi: ABIFunction,
+    arguments: Sequence[Any],
+    data: Optional[HexStr] = None,
 ) -> HexStr:
     argument_types = get_abi_input_types(abi)
 
-    if not check_if_arguments_can_be_encoded(abi, web3.codec, arguments, {}):
+    if not check_if_arguments_can_be_encoded(abi, w3.codec, arguments, {}):
         raise TypeError(
             "One or more arguments could not be encoded to the necessary "
-            "ABI type.  Expected types are: {0}".format(
-                ', '.join(argument_types),
-            )
+            f"ABI type. Expected types are: {', '.join(argument_types)}"
         )
 
     normalizers = [
-        abi_ens_resolver(web3),
         abi_address_to_hex,
         abi_bytes_to_bytes,
         abi_string_to_text,
     ]
+    if not w3.eth.is_async:
+        normalizers.append(abi_ens_resolver(w3))
+
     normalized_arguments = map_abi_data(
         normalizers,
         argument_types,
         arguments,
     )
-    encoded_arguments = web3.codec.encode(
+    encoded_arguments = w3.codec.encode(
         argument_types,
         normalized_arguments,
     )
-
     if data:
         return to_hex(HexBytes(data) + encoded_arguments)
     else:
@@ -203,7 +247,7 @@ def encode_abi(
 
 def prepare_transaction(
     address: ChecksumAddress,
-    web3: "Web3",
+    w3: Union["AsyncWeb3", "Web3"],
     fn_identifier: Union[str, Type[FallbackFn], Type[ReceiveFn]],
     contract_abi: Optional[ABI] = None,
     fn_abi: Optional[ABIFunction] = None,
@@ -218,7 +262,9 @@ def prepare_transaction(
     TODO: add new prepare_deploy_transaction API
     """
     if fn_abi is None:
-        fn_abi = find_matching_fn_abi(contract_abi, web3.codec, fn_identifier, fn_args, fn_kwargs)
+        fn_abi = find_matching_fn_abi(
+            contract_abi, w3.codec, fn_identifier, fn_args, fn_kwargs
+        )
 
     validate_payable(transaction, fn_abi)
 
@@ -227,14 +273,14 @@ def prepare_transaction(
     else:
         prepared_transaction = cast(TxParams, dict(**transaction))
 
-    if 'data' in prepared_transaction:
+    if "data" in prepared_transaction:
         raise ValueError("Transaction parameter may not contain a 'data' key")
 
     if address:
-        prepared_transaction.setdefault('to', address)
+        prepared_transaction.setdefault("to", address)
 
-    prepared_transaction['data'] = encode_transaction_data(
-        web3,
+    prepared_transaction["data"] = encode_transaction_data(
+        w3,
         fn_identifier,
         contract_abi,
         fn_abi,
@@ -245,26 +291,50 @@ def prepare_transaction(
 
 
 def encode_transaction_data(
-    web3: "Web3",
+    w3: Union["AsyncWeb3", "Web3"],
     fn_identifier: Union[str, Type[FallbackFn], Type[ReceiveFn]],
     contract_abi: Optional[ABI] = None,
     fn_abi: Optional[ABIFunction] = None,
     args: Optional[Sequence[Any]] = None,
-    kwargs: Optional[Any] = None
+    kwargs: Optional[Any] = None,
 ) -> HexStr:
     if fn_identifier is FallbackFn:
-        fn_abi, fn_selector, fn_arguments = get_fallback_function_info(contract_abi, fn_abi)
+        fn_abi, fn_selector, fn_arguments = get_fallback_function_info(
+            contract_abi, fn_abi
+        )
     elif fn_identifier is ReceiveFn:
-        fn_abi, fn_selector, fn_arguments = get_receive_function_info(contract_abi, fn_abi)
+        fn_abi, fn_selector, fn_arguments = get_receive_function_info(
+            contract_abi, fn_abi
+        )
     elif is_text(fn_identifier):
         fn_abi, fn_selector, fn_arguments = get_function_info(
             # type ignored b/c fn_id here is always str b/c FallbackFn is handled above
-            fn_identifier, web3.codec, contract_abi, fn_abi, args, kwargs,  # type: ignore
+            fn_identifier,  # type: ignore
+            w3.codec,
+            contract_abi,
+            fn_abi,
+            args,
+            kwargs,
         )
     else:
         raise TypeError("Unsupported function identifier")
 
-    return add_0x_prefix(encode_abi(web3, fn_abi, fn_arguments, fn_selector))
+    return add_0x_prefix(encode_abi(w3, fn_abi, fn_arguments, fn_selector))
+
+
+def decode_transaction_data(
+    fn_abi: ABIFunction,
+    data: HexStr,
+    normalizers: Sequence[Callable[[TypeStr, Any], Tuple[TypeStr, Any]]] = None,
+) -> Dict[str, Any]:
+    # type ignored b/c expects data arg to be HexBytes
+    data = HexBytes(data)  # type: ignore
+    types = get_abi_input_types(fn_abi)
+    abi_codec = ABICodec(default_registry)
+    decoded = abi_codec.decode(types, HexBytes(data[4:]))
+    if normalizers:
+        decoded = map_abi_data(normalizers, types, decoded)
+    return named_tree(fn_abi["inputs"], decoded)
 
 
 def get_fallback_function_info(
@@ -272,7 +342,7 @@ def get_fallback_function_info(
 ) -> Tuple[ABIFunction, HexStr, Tuple[Any, ...]]:
     if fn_abi is None:
         fn_abi = get_fallback_func_abi(contract_abi)
-    fn_selector = encode_hex(b'')
+    fn_selector = encode_hex(b"")
     fn_arguments: Tuple[Any, ...] = tuple()
     return fn_abi, fn_selector, fn_arguments
 
@@ -282,7 +352,7 @@ def get_receive_function_info(
 ) -> Tuple[ABIFunction, HexStr, Tuple[Any, ...]]:
     if fn_abi is None:
         fn_abi = get_receive_func_abi(contract_abi)
-    fn_selector = encode_hex(b'')
+    fn_selector = encode_hex(b"")
     fn_arguments: Tuple[Any, ...] = tuple()
     return fn_abi, fn_selector, fn_arguments
 
@@ -315,14 +385,103 @@ def get_function_info(
 
 
 def validate_payable(transaction: TxParams, abi: ABIFunction) -> None:
-    """Raise ValidationError if non-zero ether
+    """Raise Web3ValidationError if non-zero ether
     is sent to a non-payable function.
     """
-    if 'value' in transaction:
-        if to_integer_if_hex(transaction['value']) != 0:
-            if "payable" in abi and not abi["payable"]:
-                raise ValidationError(
+    if "value" in transaction:
+        if to_integer_if_hex(transaction["value"]) != 0:
+            if (
+                "payable" in abi
+                and not abi["payable"]
+                or "stateMutability" in abi
+                and abi["stateMutability"] == "nonpayable"
+            ):
+                raise Web3ValidationError(
                     "Sending non-zero ether to a contract function "
                     "with payable=False. Please ensure that "
                     "transaction's value is 0."
                 )
+
+
+def _get_argument_readable_type(arg: Any) -> str:
+    if is_checksum_address(arg) or is_binary_address(arg):
+        return "address"
+
+    return arg.__class__.__name__
+
+
+def parse_block_identifier(
+    w3: "Web3", block_identifier: BlockIdentifier
+) -> BlockIdentifier:
+    if block_identifier is None:
+        return w3.eth.default_block
+    if isinstance(block_identifier, int):
+        return parse_block_identifier_int(w3, block_identifier)
+    elif block_identifier in ["latest", "earliest", "pending", "safe", "finalized"]:
+        return block_identifier
+    elif isinstance(block_identifier, bytes) or is_hex_encoded_block_hash(
+        block_identifier
+    ):
+        return w3.eth.get_block(block_identifier)["number"]
+    else:
+        raise BlockNumberOutofRange
+
+
+def parse_block_identifier_int(w3: "Web3", block_identifier_int: int) -> BlockNumber:
+    if block_identifier_int >= 0:
+        block_num = block_identifier_int
+    else:
+        last_block = w3.eth.get_block("latest")["number"]
+        block_num = last_block + block_identifier_int + 1
+        if block_num < 0:
+            raise BlockNumberOutofRange
+    return BlockNumber(block_num)
+
+
+def parse_block_identifier_no_extra_call(
+    w3: Union["Web3", "AsyncWeb3"], block_identifier: BlockIdentifier
+) -> BlockIdentifier:
+    if block_identifier is None:
+        return w3.eth.default_block
+    elif isinstance(block_identifier, int) and block_identifier >= 0:
+        return block_identifier
+    elif block_identifier in ["latest", "earliest", "pending", "safe", "finalized"]:
+        return block_identifier
+    elif isinstance(block_identifier, bytes):
+        return HexBytes(block_identifier)
+    elif is_hex_encoded_block_hash(block_identifier):
+        return HexStr(str(block_identifier))
+    else:
+        raise BlockNumberOutofRange
+
+
+async def async_parse_block_identifier(
+    async_w3: "AsyncWeb3", block_identifier: BlockIdentifier
+) -> BlockIdentifier:
+    if block_identifier is None:
+        return async_w3.eth.default_block
+    if isinstance(block_identifier, int):
+        return await async_parse_block_identifier_int(async_w3, block_identifier)
+    elif block_identifier in ["latest", "earliest", "pending", "safe", "finalized"]:
+        return block_identifier
+    elif isinstance(block_identifier, bytes) or is_hex_encoded_block_hash(
+        block_identifier
+    ):
+        requested_block = await async_w3.eth.get_block(block_identifier)
+        return requested_block["number"]
+    else:
+        raise BlockNumberOutofRange
+
+
+async def async_parse_block_identifier_int(
+    async_w3: "AsyncWeb3", block_identifier_int: int
+) -> BlockNumber:
+    if block_identifier_int >= 0:
+        block_num = block_identifier_int
+    else:
+        last_block = await async_w3.eth.get_block("latest")
+        last_block_num = last_block["number"]
+        block_num = last_block_num + block_identifier_int + 1
+        if block_num < 0:
+            raise BlockNumberOutofRange
+    return BlockNumber(block_num)
