@@ -4,9 +4,6 @@ from typing import (
     NoReturn,
     Sequence,
 )
-from unittest.mock import (
-    patch,
-)
 
 from eth_typing import (
     ChecksumAddress,
@@ -18,7 +15,9 @@ from hexbytes import (
     HexBytes,
 )
 
-from web3 import Web3
+from web3 import (
+    Web3,
+)
 from web3._utils.ens import (
     ens_addresses,
 )
@@ -28,8 +27,8 @@ from web3.exceptions import (
 
 
 class Web3ModuleTest:
-    def test_web3_client_version(self, web3: Web3) -> None:
-        client_version = web3.client_version
+    def test_web3_client_version(self, w3: Web3) -> None:
+        client_version = w3.client_version
         self._check_web3_client_version(client_version)
 
     def _check_web3_client_version(self, client_version: str) -> NoReturn:
@@ -224,17 +223,17 @@ class Web3ModuleTest:
     )
     def test_solidity_keccak(
         self,
-        web3: "Web3",
+        w3: "Web3",
         types: Sequence[TypeStr],
         values: Sequence[Any],
         expected: HexBytes,
     ) -> None:
         if isinstance(expected, type) and issubclass(expected, Exception):
-            with pytest.raises(expected):
-                web3.solidity_keccak(types, values)
+            with pytest.raises(expected):  # type: ignore
+                w3.solidity_keccak(types, values)
             return
 
-        actual = web3.solidity_keccak(types, values)
+        actual = w3.solidity_keccak(types, values)
         assert actual == expected
 
     @pytest.mark.parametrize(
@@ -258,13 +257,13 @@ class Web3ModuleTest:
     )
     def test_solidity_keccak_ens(
         self,
-        web3: "Web3",
+        w3: "Web3",
         types: Sequence[TypeStr],
         values: Sequence[str],
         expected: HexBytes,
     ) -> None:
         with ens_addresses(
-            web3,
+            w3,
             {
                 "one.eth": ChecksumAddress(
                     HexAddress(HexStr("0x49EdDD3769c0712032808D86597B84ac5c2F5614"))
@@ -279,7 +278,7 @@ class Web3ModuleTest:
                 Web3.solidity_keccak(types, values)
 
             # when called as instance method, ens lookups can succeed
-            actual = web3.solidity_keccak(types, values)
+            actual = w3.solidity_keccak(types, values)
             assert actual == expected
 
     @pytest.mark.parametrize(
@@ -291,52 +290,10 @@ class Web3ModuleTest:
         ),
     )
     def test_solidity_keccak_same_number_of_types_and_values(
-        self, web3: "Web3", types: Sequence[TypeStr], values: Sequence[Any]
+        self, w3: "Web3", types: Sequence[TypeStr], values: Sequence[Any]
     ) -> None:
         with pytest.raises(ValueError):
-            web3.solidity_keccak(types, values)
+            w3.solidity_keccak(types, values)
 
-    @patch("web3.main.Web3.solidity_keccak", lambda w3, *args, **kwargs: (args, kwargs))
-    def test_solidityKeccak_calls_solidity_keccak_with_deprecation_warning(
-        self,
-        web3: "Web3",
-    ) -> None:
-        # This should test that the arguments are correctly passed on to
-        # Web3.solidity_keccak and, in turn, should pass all tests related to
-        # solidity_keccak.
-        with pytest.warns(DeprecationWarning):
-            passed_on_args, _ = web3.solidityKeccak(["bool"], [True])
-            assert passed_on_args == (["bool"], [True])
-
-    @patch("web3.main.Web3.solidity_keccak", lambda w3, *args, **kwargs: (args, kwargs))
-    def test_soliditySha3_calls_solidity_keccak_with_deprecation_warning(
-        self,
-        web3: "Web3",
-    ) -> None:
-        # This should test that the arguments are correctly passed on to
-        # Web3.solidity_keccak and, in turn, should pass all tests related to
-        # solidity_keccak.
-        with pytest.warns(DeprecationWarning):
-            passed_on_args, _ = web3.soliditySha3(["bool"], [True])
-            assert passed_on_args == (["bool"], [True])
-
-    def test_is_connected(self, web3: "Web3") -> None:
-        assert web3.is_connected()
-
-    #
-    # Deprecated
-    #
-    def test_isConnected(self, web3: "Web3") -> None:
-        with pytest.warns(
-            DeprecationWarning,
-            match="isConnected is deprecated in favor of is_connected",
-        ):
-            assert web3.isConnected()
-
-    def test_web3_clientVersion(self, web3: Web3) -> None:
-        with pytest.warns(
-            DeprecationWarning,
-            match="clientVersion is deprecated in favor of client_version"
-        ):
-            client_version = web3.clientVersion
-        self._check_web3_client_version(client_version)
+    def test_is_connected(self, w3: "Web3") -> None:
+        assert w3.is_connected()
