@@ -54,18 +54,19 @@ from web3._utils.abi import (
 from web3.exceptions import (
     InvalidAddress,
 )
-from web3.types import (  # noqa: F401
+from web3.types import (
     ABI,
-    ABIEvent,
     ABIFunction,
 )
 
 
 def _prepare_selector_collision_msg(duplicates: Dict[HexStr, ABIFunction]) -> str:
     dup_sel = valmap(apply_formatter_to_array(abi_to_signature), duplicates)
-    joined_funcs = valmap(lambda funcs: ', '.join(funcs), dup_sel)
-    func_sel_msg_list = [funcs + ' have selector ' + sel for sel, funcs in joined_funcs.items()]
-    return ' and\n'.join(func_sel_msg_list)
+    joined_funcs = valmap(lambda funcs: ", ".join(funcs), dup_sel)
+    func_sel_msg_list = [
+        funcs + " have selector " + sel for sel, funcs in joined_funcs.items()
+    ]
+    return " and\n".join(func_sel_msg_list)
 
 
 def validate_abi(abi: ABI) -> None:
@@ -78,16 +79,13 @@ def validate_abi(abi: ABI) -> None:
     if not all(is_dict(e) for e in abi):
         raise ValueError("'abi' is not a list of dictionaries")
 
-    functions = filter_by_type('function', abi)
-    selectors = groupby(
-        compose(encode_hex, function_abi_to_4byte_selector),
-        functions
-    )
+    functions = filter_by_type("function", abi)
+    selectors = groupby(compose(encode_hex, function_abi_to_4byte_selector), functions)
     duplicates = valfilter(lambda funcs: len(funcs) > 1, selectors)
     if duplicates:
         raise ValueError(
-            'Abi contains functions with colliding selectors. '
-            'Functions {0}'.format(_prepare_selector_collision_msg(duplicates))
+            "Abi contains functions with colliding selectors. "
+            f"Functions {_prepare_selector_collision_msg(duplicates)}"
         )
 
 
@@ -96,7 +94,7 @@ def validate_abi_type(abi_type: TypeStr) -> None:
     Helper function for validating an abi_type
     """
     if not is_recognized_type(abi_type):
-        raise ValueError("Unrecognized abi_type: {abi_type}".format(abi_type=abi_type))
+        raise ValueError(f"Unrecognized abi_type: {abi_type}")
 
 
 def validate_abi_value(abi_type: TypeStr, value: Any) -> None:
@@ -110,15 +108,13 @@ def validate_abi_value(abi_type: TypeStr, value: Any) -> None:
         if specified_length is not None:
             if specified_length < 1:
                 raise TypeError(
-                    "Invalid abi-type: {abi_type}. Length of fixed sized arrays"
-                    "must be greater than 0."
-                    .format(abi_type=abi_type)
+                    f"Invalid abi-type: {abi_type}. Length of fixed sized "
+                    "arrays must be greater than 0."
                 )
             if specified_length != len(value):
                 raise TypeError(
                     "The following array length does not the length specified"
-                    "by the abi-type, {abi_type}: {value}"
-                    .format(abi_type=abi_type, value=value)
+                    f"by the abi-type, {abi_type}: {value}"
                 )
 
         # validate sub_types
@@ -149,15 +145,16 @@ def validate_abi_value(abi_type: TypeStr, value: Any) -> None:
     elif is_string_type(abi_type) and is_string(value):
         return
 
-    raise TypeError(
-        "The following abi value is not a '{abi_type}': {value}"
-        .format(abi_type=abi_type, value=value)
-    )
+    raise TypeError(f"The following abi value is not a '{abi_type}': {value}")
 
 
 def is_not_address_string(value: Any) -> bool:
-    return (is_string(value) and not is_bytes(value) and not
-            is_checksum_address(value) and not is_hex_address(value))
+    return (
+        is_string(value)
+        and not is_bytes(value)
+        and not is_checksum_address(value)
+        and not is_hex_address(value)
+    )
 
 
 def validate_address(value: Any) -> None:
@@ -170,19 +167,23 @@ def validate_address(value: Any) -> None:
         return
     if is_bytes(value):
         if not is_binary_address(value):
-            raise InvalidAddress("Address must be 20 bytes when input type is bytes", value)
+            raise InvalidAddress(
+                "Address must be 20 bytes when input type is bytes", value
+            )
         return
 
     if not isinstance(value, str):
-        raise TypeError('Address {} must be provided as a string'.format(value))
+        raise TypeError(f"Address {value} must be provided as a string")
     if not is_hex_address(value):
-        raise InvalidAddress("Address must be 20 bytes, as a hex string with a 0x prefix", value)
+        raise InvalidAddress(
+            "Address must be 20 bytes, as a hex string with a 0x prefix", value
+        )
     if not is_checksum_address(value):
         if value == value.lower():
             raise InvalidAddress(
-                "Web3.py only accepts checksum addresses. "
-                "The software that gave you this non-checksum address should be considered unsafe, "
-                "please file it as a bug on their platform. "
+                "web3.py only accepts checksum addresses. "
+                "The software that gave you this non-checksum address should be "
+                "considered unsafe, please file it as a bug on their platform. "
                 "Try using an ENS name instead. Or, if you must accept lower safety, "
                 "use Web3.to_checksum_address(lower_case_address).",
                 value,
@@ -205,5 +206,5 @@ def assert_one_val(*args: Any, **kwargs: Any) -> None:
     if not has_one_val(*args, **kwargs):
         raise TypeError(
             "Exactly one of the passed values can be specified. "
-            "Instead, values were: %r, %r" % (args, kwargs)
+            f"Instead, values were: {args!r}, {kwargs!r}"
         )
