@@ -1,5 +1,6 @@
 import functools
 from typing import (
+    Coroutine,
     TYPE_CHECKING,
     Any,
     Callable,
@@ -7,14 +8,20 @@ from typing import (
 )
 
 from web3.types import (
+    AsyncMiddleware,
+    Middleware,
     RPCEndpoint,
     RPCResponse,
-    Middleware,
 )
+
 from .abi import (  # noqa: F401
     abi_middleware,
 )
+from .async_cache import (  # noqa: F401
+    _async_simple_cache_middleware as async_simple_cache_middleware,
+)
 from .attrdict import (  # noqa: F401
+    async_attrdict_middleware,
     attrdict_middleware,
 )
 from .buffered_gas_estimate import (  # noqa: F401
@@ -36,9 +43,12 @@ from .exception_retry_request import (  # noqa: F401
     http_retry_request_middleware,
 )
 from .filter import (  # noqa: F401
+    async_local_filter_middleware,
     local_filter_middleware,
 )
 from .fixture import (  # noqa: F401
+    async_construct_error_generator_middleware,
+    async_construct_result_generator_middleware,
     construct_error_generator_middleware,
     construct_fixture_middleware,
     construct_result_generator_middleware,
@@ -67,6 +77,7 @@ from .signing import (  # noqa: F401
     construct_sign_and_send_raw_middleware,
 )
 from .stalecheck import (  # noqa: F401
+    async_make_stalecheck_middleware,
     make_stalecheck_middleware,
 )
 from .validation import (  # noqa: F401
@@ -75,43 +86,45 @@ from .validation import (  # noqa: F401
 )
 
 if TYPE_CHECKING:
-    from web3 import Web3  # noqa: F401
+    from web3 import AsyncWeb3, Web3  # noqa: F401
 
 
 def combine_middlewares(
     middlewares: Sequence[Middleware],
-    web3: 'Web3',
-    provider_request_fn: Callable[[RPCEndpoint, Any], Any]
+    w3: "Web3",
+    provider_request_fn: Callable[[RPCEndpoint, Any], Any],
 ) -> Callable[..., RPCResponse]:
     """
     Returns a callable function which will call the provider.provider_request
     function wrapped with all of the middlewares.
     """
     return functools.reduce(
-        lambda request_fn, middleware: middleware(request_fn, web3),
+        lambda request_fn, middleware: middleware(request_fn, w3),
         reversed(middlewares),
         provider_request_fn,
     )
 
 
 async def async_combine_middlewares(
-    middlewares: Sequence[Middleware],
-    web3: 'Web3',
-    provider_request_fn: Callable[[RPCEndpoint, Any], Any]
-) -> Callable[..., RPCResponse]:
+    middlewares: Sequence[AsyncMiddleware],
+    async_w3: "AsyncWeb3",
+    provider_request_fn: Callable[[RPCEndpoint, Any], Any],
+) -> Callable[..., Coroutine[Any, Any, RPCResponse]]:
     """
     Returns a callable function which will call the provider.provider_request
     function wrapped with all of the middlewares.
     """
     accumulator_fn = provider_request_fn
     for middleware in reversed(middlewares):
-        accumulator_fn = await construct_middleware(middleware, accumulator_fn, web3)
+        accumulator_fn = await construct_middleware(
+            middleware, accumulator_fn, async_w3
+        )
     return accumulator_fn
 
 
 async def construct_middleware(
-    middleware: Middleware,
+    async_middleware: AsyncMiddleware,
     fn: Callable[..., RPCResponse],
-    w3: 'Web3'
-) -> Callable[[RPCEndpoint, Any], Any]:
-    return await middleware(fn, w3)
+    async_w3: "AsyncWeb3",
+) -> Callable[[RPCEndpoint, Any], RPCResponse]:
+    return await async_middleware(fn, async_w3)
