@@ -8,7 +8,6 @@ from typing import (
     Collection,
     Dict,
     Set,
-    Type,
     cast,
 )
 
@@ -24,189 +23,119 @@ from web3._utils.compat import (
     Literal,
     TypedDict,
 )
-from web3.types import (  # noqa: F401
+from web3.types import (
     BlockData,
     BlockNumber,
     Middleware,
     RPCEndpoint,
     RPCResponse,
 )
+from web3.utils.caching import (
+    SimpleCache,
+)
 
 if TYPE_CHECKING:
     from web3 import Web3  # noqa: F401
 
-
-SIMPLE_CACHE_RPC_WHITELIST = cast(Set[RPCEndpoint], {
-    'web3_clientVersion',
-    'web3_sha3',
-    'net_version',
-    # 'net_peerCount',
-    # 'net_listening',
-    'eth_protocolVersion',
-    # 'eth_syncing',
-    # 'eth_coinbase',
-    # 'eth_mining',
-    # 'eth_hashrate',
-    # 'eth_gasPrice',
-    # 'eth_accounts',
-    # 'eth_blockNumber',
-    # 'eth_getBalance',
-    # 'eth_getStorageAt',
-    # 'eth_getTransactionCount',
-    'eth_getBlockTransactionCountByHash',
-    # 'eth_getBlockTransactionCountByNumber',
-    'eth_getUncleCountByBlockHash',
-    # 'eth_getUncleCountByBlockNumber',
-    # 'eth_getCode',
-    # 'eth_sign',
-    # 'eth_sendTransaction',
-    # 'eth_sendRawTransaction',
-    # 'eth_call',
-    # 'eth_estimateGas',
-    'eth_getBlockByHash',
-    # 'eth_getBlockByNumber',
-    'eth_getTransactionByHash',
-    'eth_getTransactionByBlockHashAndIndex',
-    # 'eth_getTransactionByBlockNumberAndIndex',
-    # 'eth_getTransactionReceipt',
-    'eth_getRawTransactionByHash',
-    'eth_getUncleByBlockHashAndIndex',
-    # 'eth_getUncleByBlockNumberAndIndex',
-    # 'eth_getCompilers',
-    # 'eth_compileLLL',
-    # 'eth_compileSolidity',
-    # 'eth_compileSerpent',
-    # 'eth_newFilter',
-    # 'eth_newBlockFilter',
-    # 'eth_newPendingTransactionFilter',
-    # 'eth_uninstallFilter',
-    # 'eth_getFilterChanges',
-    # 'eth_getFilterLogs',
-    # 'eth_getLogs',
-    # 'eth_getWork',
-    # 'eth_submitWork',
-    # 'eth_submitHashrate',
-    'eth_chainId',
-})
+SIMPLE_CACHE_RPC_WHITELIST = cast(
+    Set[RPCEndpoint],
+    (
+        "web3_clientVersion",
+        "net_version",
+        "eth_getBlockTransactionCountByHash",
+        "eth_getUncleCountByBlockHash",
+        "eth_getBlockByHash",
+        "eth_getTransactionByHash",
+        "eth_getTransactionByBlockHashAndIndex",
+        "eth_getRawTransactionByHash",
+        "eth_getUncleByBlockHashAndIndex",
+        "eth_chainId",
+    ),
+)
 
 
-def _should_cache(method: RPCEndpoint, params: Any, response: RPCResponse) -> bool:
-    if 'error' in response:
-        return False
-    elif 'result' not in response:
-        return False
-
-    if response['result'] is None:
-        return False
-    return True
+def _should_cache_response(
+    _method: RPCEndpoint, _params: Any, response: RPCResponse
+) -> bool:
+    return (
+        "error" not in response
+        and "result" in response
+        and response["result"] is not None
+    )
 
 
 def construct_simple_cache_middleware(
-    cache_class: Type[Dict[Any, Any]],
-    rpc_whitelist: Collection[RPCEndpoint] = SIMPLE_CACHE_RPC_WHITELIST,
-    should_cache_fn: Callable[[RPCEndpoint, Any, RPCResponse], bool] = _should_cache
+    cache: SimpleCache = None,
+    rpc_whitelist: Collection[RPCEndpoint] = None,
+    should_cache_fn: Callable[
+        [RPCEndpoint, Any, RPCResponse], bool
+    ] = _should_cache_response,
 ) -> Middleware:
     """
     Constructs a middleware which caches responses based on the request
     ``method`` and ``params``
 
-    :param cache_class: Any dictionary-like object
+    :param cache: A ``SimpleCache`` class.
     :param rpc_whitelist: A set of RPC methods which may have their responses cached.
     :param should_cache_fn: A callable which accepts ``method`` ``params`` and
         ``response`` and returns a boolean as to whether the response should be
         cached.
     """
+    if cache is None:
+        cache = SimpleCache(256)
+
+    if rpc_whitelist is None:
+        rpc_whitelist = SIMPLE_CACHE_RPC_WHITELIST
+
     def simple_cache_middleware(
-        make_request: Callable[[RPCEndpoint, Any], RPCResponse], web3: "Web3"
+        make_request: Callable[[RPCEndpoint, Any], RPCResponse], _w3: "Web3"
     ) -> Callable[[RPCEndpoint, Any], RPCResponse]:
-        cache = cache_class()
         lock = threading.Lock()
 
-        def middleware(
-            method: RPCEndpoint, params: Any
-        ) -> RPCResponse:
-            lock_acquired = lock.acquire(blocking=False) if method in rpc_whitelist else False
+        def middleware(method: RPCEndpoint, params: Any) -> RPCResponse:
+            if method in rpc_whitelist:
+                cache_key = generate_cache_key(
+                    f"{threading.get_ident()}:{(method, params)}"
+                )
+                cached_request = cache.get_cache_entry(cache_key)
+                if cached_request is not None:
+                    return cached_request
+
+                response = make_request(method, params)
+                if should_cache_fn(method, params, response):
+                    lock.acquire(blocking=False)
+                    try:
+                        cache.cache(cache_key, response)
+                    finally:
+                        lock.release()
+                return response
+            else:
+                return make_request(method, params)
 
-            try:
-                if lock_acquired and method in rpc_whitelist:
-                    cache_key = generate_cache_key((method, params))
-                    if cache_key not in cache:
-                        response = make_request(method, params)
-                        if should_cache_fn(method, params, response):
-                            cache[cache_key] = response
-                        return response
-                    return cache[cache_key]
-                else:
-                    return make_request(method, params)
-            finally:
-                if lock_acquired:
-                    lock.release()
         return middleware
+
     return simple_cache_middleware
 
 
-_simple_cache_middleware = construct_simple_cache_middleware(
-    cache_class=cast(Type[Dict[Any, Any]], functools.partial(lru.LRU, 256)),
+_simple_cache_middleware = construct_simple_cache_middleware()
+
+
+TIME_BASED_CACHE_RPC_WHITELIST = cast(
+    Set[RPCEndpoint],
+    {
+        "eth_coinbase",
+        "eth_accounts",
+    },
 )
 
 
-TIME_BASED_CACHE_RPC_WHITELIST = cast(Set[RPCEndpoint], {
-    # 'web3_clientVersion',
-    # 'web3_sha3',
-    # 'net_version',
-    # 'net_peerCount',
-    # 'net_listening',
-    # 'eth_protocolVersion',
-    # 'eth_syncing',
-    'eth_coinbase',
-    # 'eth_mining',
-    # 'eth_hashrate',
-    # 'eth_gasPrice',
-    'eth_accounts',
-    # 'eth_blockNumber',
-    # 'eth_getBalance',
-    # 'eth_getStorageAt',
-    # 'eth_getTransactionCount',
-    # 'eth_getBlockTransactionCountByHash',
-    # 'eth_getBlockTransactionCountByNumber',
-    # 'eth_getUncleCountByBlockHash',
-    # 'eth_getUncleCountByBlockNumber',
-    # 'eth_getCode',
-    # 'eth_sign',
-    # 'eth_sendTransaction',
-    # 'eth_sendRawTransaction',
-    # 'eth_call',
-    # 'eth_estimateGas',
-    # 'eth_getBlockByHash',
-    # 'eth_getBlockByNumber',
-    # 'eth_getTransactionByHash',
-    # 'eth_getTransactionByBlockHashAndIndex',
-    # 'eth_getTransactionByBlockNumberAndIndex',
-    # 'eth_getTransactionReceipt',
-    # 'eth_getUncleByBlockHashAndIndex',
-    # 'eth_getUncleByBlockNumberAndIndex',
-    # 'eth_getCompilers',
-    # 'eth_compileLLL',
-    # 'eth_compileSolidity',
-    # 'eth_compileSerpent',
-    # 'eth_newFilter',
-    # 'eth_newBlockFilter',
-    # 'eth_newPendingTransactionFilter',
-    # 'eth_uninstallFilter',
-    # 'eth_getFilterChanges',
-    # 'eth_getFilterLogs',
-    # 'eth_getLogs',
-    # 'eth_getWork',
-    # 'eth_submitWork',
-    # 'eth_submitHashrate',
-})
-
-
 def construct_time_based_cache_middleware(
     cache_class: Callable[..., Dict[Any, Any]],
     cache_expire_seconds: int = 15,
     rpc_whitelist: Collection[RPCEndpoint] = TIME_BASED_CACHE_RPC_WHITELIST,
-    should_cache_fn: Callable[[RPCEndpoint, Any, RPCResponse], bool] = _should_cache
+    should_cache_fn: Callable[
+        [RPCEndpoint, Any, RPCResponse], bool
+    ] = _should_cache_response,
 ) -> Middleware:
     """
     Constructs a middleware which caches responses based on the request
@@ -220,14 +149,17 @@ def construct_time_based_cache_middleware(
         ``response`` and returns a boolean as to whether the response should be
         cached.
     """
+
     def time_based_cache_middleware(
-        make_request: Callable[[RPCEndpoint, Any], Any], web3: "Web3"
+        make_request: Callable[[RPCEndpoint, Any], Any], w3: "Web3"
     ) -> Callable[[RPCEndpoint, Any], RPCResponse]:
         cache = cache_class()
         lock = threading.Lock()
 
         def middleware(method: RPCEndpoint, params: Any) -> RPCResponse:
-            lock_acquired = lock.acquire(blocking=False) if method in rpc_whitelist else False
+            lock_acquired = (
+                lock.acquire(blocking=False) if method in rpc_whitelist else False
+            )
 
             try:
                 if lock_acquired and method in rpc_whitelist:
@@ -254,7 +186,9 @@ def construct_time_based_cache_middleware(
             finally:
                 if lock_acquired:
                     lock.release()
+
         return middleware
+
     return time_based_cache_middleware
 
 
@@ -263,76 +197,52 @@ _time_based_cache_middleware = construct_time_based_cache_middleware(
 )
 
 
-BLOCK_NUMBER_RPC_WHITELIST = cast(Set[RPCEndpoint], {
-    # 'web3_clientVersion',
-    # 'web3_sha3',
-    # 'net_version',
-    # 'net_peerCount',
-    # 'net_listening',
-    # 'eth_protocolVersion',
-    # 'eth_syncing',
-    # 'eth_coinbase',
-    # 'eth_mining',
-    # 'eth_hashrate',
-    'eth_gasPrice',
-    # 'eth_accounts',
-    'eth_blockNumber',
-    'eth_getBalance',
-    'eth_getStorageAt',
-    'eth_getTransactionCount',
-    # 'eth_getBlockTransactionCountByHash',
-    'eth_getBlockTransactionCountByNumber',
-    # 'eth_getUncleCountByBlockHash',
-    'eth_getUncleCountByBlockNumber',
-    'eth_getCode',
-    # 'eth_sign',
-    # 'eth_sendTransaction',
-    # 'eth_sendRawTransaction',
-    'eth_call',
-    'eth_estimateGas',
-    # 'eth_getBlockByHash',
-    'eth_getBlockByNumber',
-    # 'eth_getTransactionByHash',
-    # 'eth_getTransactionByBlockHashAndIndex',
-    'eth_getTransactionByBlockNumberAndIndex',
-    'eth_getTransactionReceipt',
-    # 'eth_getUncleByBlockHashAndIndex',
-    'eth_getUncleByBlockNumberAndIndex',
-    # 'eth_getCompilers',
-    # 'eth_compileLLL',
-    # 'eth_compileSolidity',
-    # 'eth_compileSerpent',
-    # 'eth_newFilter',
-    # 'eth_newBlockFilter',
-    # 'eth_newPendingTransactionFilter',
-    # 'eth_uninstallFilter',
-    # 'eth_getFilterChanges',
-    # 'eth_getFilterLogs',
-    'eth_getLogs',
-    # 'eth_getWork',
-    # 'eth_submitWork',
-    # 'eth_submitHashrate',
-})
+BLOCK_NUMBER_RPC_WHITELIST = cast(
+    Set[RPCEndpoint],
+    {
+        "eth_gasPrice",
+        "eth_blockNumber",
+        "eth_getBalance",
+        "eth_getStorageAt",
+        "eth_getTransactionCount",
+        "eth_getBlockTransactionCountByNumber",
+        "eth_getUncleCountByBlockNumber",
+        "eth_getCode",
+        "eth_call",
+        "eth_estimateGas",
+        "eth_getBlockByNumber",
+        "eth_getTransactionByBlockNumberAndIndex",
+        "eth_getTransactionReceipt",
+        "eth_getUncleByBlockNumberAndIndex",
+        "eth_getLogs",
+    },
+)
 
-AVG_BLOCK_TIME_KEY: Literal['avg_block_time'] = 'avg_block_time'
-AVG_BLOCK_SAMPLE_SIZE_KEY: Literal['avg_block_sample_size'] = 'avg_block_sample_size'
-AVG_BLOCK_TIME_UPDATED_AT_KEY: Literal['avg_block_time_updated_at'] = 'avg_block_time_updated_at'
+AVG_BLOCK_TIME_KEY: Literal["avg_block_time"] = "avg_block_time"
+AVG_BLOCK_SAMPLE_SIZE_KEY: Literal["avg_block_sample_size"] = "avg_block_sample_size"
+AVG_BLOCK_TIME_UPDATED_AT_KEY: Literal[
+    "avg_block_time_updated_at"
+] = "avg_block_time_updated_at"
 
 
 def _is_latest_block_number_request(method: RPCEndpoint, params: Any) -> bool:
-    if method != 'eth_getBlockByNumber':
+    if method != "eth_getBlockByNumber":
         return False
-    elif is_list_like(params) and tuple(params[:1]) == ('latest',):
+    elif is_list_like(params) and tuple(params[:1]) == ("latest",):
         return True
     return False
 
 
-BlockInfoCache = TypedDict("BlockInfoCache", {
-    "avg_block_time": float,
-    "avg_block_sample_size": int,
-    "avg_block_time_updated_at": float,
-    "latest_block": BlockData,
-}, total=False)
+BlockInfoCache = TypedDict(
+    "BlockInfoCache",
+    {
+        "avg_block_time": float,
+        "avg_block_sample_size": int,
+        "avg_block_time_updated_at": float,
+        "latest_block": BlockData,
+    },
+    total=False,
+)
 
 
 def construct_latest_block_based_cache_middleware(
@@ -340,7 +250,9 @@ def construct_latest_block_based_cache_middleware(
     rpc_whitelist: Collection[RPCEndpoint] = BLOCK_NUMBER_RPC_WHITELIST,
     average_block_time_sample_size: int = 240,
     default_average_block_time: int = 15,
-    should_cache_fn: Callable[[RPCEndpoint, Any, RPCResponse], bool] = _should_cache
+    should_cache_fn: Callable[
+        [RPCEndpoint, Any, RPCResponse], bool
+    ] = _should_cache_response,
 ) -> Middleware:
     """
     Constructs a middleware which caches responses based on the request
@@ -348,6 +260,9 @@ def construct_latest_block_based_cache_middleware(
 
     :param cache_class: Any dictionary-like object
     :param rpc_whitelist: A set of RPC methods which may have their responses cached.
+    :param average_block_time_sample_size: number of blocks to look back when computing
+        average block time
+    :param default_average_block_time: estimated number of seconds per block
     :param should_cache_fn: A callable which accepts ``method`` ``params`` and
         ``response`` and returns a boolean as to whether the response should be
         cached.
@@ -358,14 +273,17 @@ def construct_latest_block_based_cache_middleware(
         a new block when the last seen latest block is older than the average
         block time.
     """
+
     def latest_block_based_cache_middleware(
-        make_request: Callable[[RPCEndpoint, Any], Any], web3: "Web3"
+        make_request: Callable[[RPCEndpoint, Any], Any], w3: "Web3"
     ) -> Callable[[RPCEndpoint, Any], RPCResponse]:
         cache = cache_class()
         block_info: BlockInfoCache = {}
 
         def _update_block_info_cache() -> None:
-            avg_block_time = block_info.get(AVG_BLOCK_TIME_KEY, default_average_block_time)
+            avg_block_time = block_info.get(
+                AVG_BLOCK_TIME_KEY, default_average_block_time
+            )
             avg_block_sample_size = block_info.get(AVG_BLOCK_SAMPLE_SIZE_KEY, 0)
             avg_block_time_updated_at = block_info.get(AVG_BLOCK_TIME_UPDATED_AT_KEY, 0)
 
@@ -374,46 +292,50 @@ def construct_latest_block_based_cache_middleware(
                 avg_block_time_age_in_blocks: float = avg_block_sample_size
             else:
                 avg_block_time_age_in_blocks = (
-                    (time.time() - avg_block_time_updated_at) / avg_block_time
-                )
+                    time.time() - avg_block_time_updated_at
+                ) / avg_block_time
 
             if avg_block_time_age_in_blocks >= avg_block_sample_size:
                 # If the length of time since the average block time as
                 # measured by blocks is greater than or equal to the number of
                 # blocks sampled then we need to recompute the average block
                 # time.
-                latest_block = web3.eth.get_block('latest')
-                ancestor_block_number = BlockNumber(max(
-                    0,
-                    latest_block['number'] - average_block_time_sample_size,
-                ))
-                ancestor_block = web3.eth.get_block(ancestor_block_number)
-                sample_size = latest_block['number'] - ancestor_block_number
+                latest_block = w3.eth.get_block("latest")
+                ancestor_block_number = BlockNumber(
+                    max(
+                        0,
+                        latest_block["number"] - average_block_time_sample_size,
+                    )
+                )
+                ancestor_block = w3.eth.get_block(ancestor_block_number)
+                sample_size = latest_block["number"] - ancestor_block_number
 
                 block_info[AVG_BLOCK_SAMPLE_SIZE_KEY] = sample_size
                 if sample_size != 0:
                     block_info[AVG_BLOCK_TIME_KEY] = (
-                        (latest_block['timestamp'] - ancestor_block['timestamp']) / sample_size
-                    )
+                        latest_block["timestamp"] - ancestor_block["timestamp"]
+                    ) / sample_size
                 else:
                     block_info[AVG_BLOCK_TIME_KEY] = avg_block_time
                 block_info[AVG_BLOCK_TIME_UPDATED_AT_KEY] = time.time()
 
-            if 'latest_block' in block_info:
-                latest_block = block_info['latest_block']
-                time_since_latest_block = time.time() - latest_block['timestamp']
+            if "latest_block" in block_info:
+                latest_block = block_info["latest_block"]
+                time_since_latest_block = time.time() - latest_block["timestamp"]
 
                 # latest block is too old so update cache
                 if time_since_latest_block > avg_block_time:
-                    block_info['latest_block'] = web3.eth.get_block('latest')
+                    block_info["latest_block"] = w3.eth.get_block("latest")
             else:
                 # latest block has not been fetched so we fetch it.
-                block_info['latest_block'] = web3.eth.get_block('latest')
+                block_info["latest_block"] = w3.eth.get_block("latest")
 
         lock = threading.Lock()
 
         def middleware(method: RPCEndpoint, params: Any) -> RPCResponse:
-            lock_acquired = lock.acquire(blocking=False) if method in rpc_whitelist else False
+            lock_acquired = (
+                lock.acquire(blocking=False) if method in rpc_whitelist else False
+            )
 
             try:
                 should_try_cache = (
@@ -423,7 +345,7 @@ def construct_latest_block_based_cache_middleware(
                 )
                 if should_try_cache:
                     _update_block_info_cache()
-                    latest_block_hash = block_info['latest_block']['hash']
+                    latest_block_hash = block_info["latest_block"]["hash"]
                     cache_key = generate_cache_key((latest_block_hash, method, params))
                     if cache_key in cache:
                         return cache[cache_key]
@@ -437,7 +359,9 @@ def construct_latest_block_based_cache_middleware(
             finally:
                 if lock_acquired:
                     lock.release()
+
         return middleware
+
     return latest_block_based_cache_middleware
 
 
