@@ -17,9 +17,6 @@ from eth_typing import (
     URI,
 )
 
-from web3._utils.decorators import (
-    deprecated_for,
-)
 from web3.exceptions import (
     CannotHandleRequest,
 )
@@ -58,16 +55,12 @@ def load_provider_from_uri(
         return WebsocketProvider(uri_string)
     else:
         raise NotImplementedError(
-            "Web3 does not know how to connect to scheme %r in %r"
-            % (
-                uri.scheme,
-                uri_string,
-            )
+            "Web3 does not know how to connect to scheme "
+            f"{uri.scheme!r} in {uri_string!r}"
         )
 
 
 class AutoProvider(BaseProvider):
-
     default_providers = (
         load_provider_from_environment,
         IPCProvider,
@@ -83,7 +76,8 @@ class AutoProvider(BaseProvider):
         ] = None,
     ) -> None:
         """
-        :param iterable potential_providers: ordered series of provider classes to attempt with
+        :param iterable potential_providers: ordered series of provider classes
+            to attempt with
 
         AutoProvider will initialize each potential provider (without arguments),
         in an attempt to find an active node. The list will default to
@@ -97,13 +91,9 @@ class AutoProvider(BaseProvider):
     def make_request(self, method: RPCEndpoint, params: Any) -> RPCResponse:
         try:
             return self._proxy_request(method, params)
-        except IOError:
+        except OSError:
             return self._proxy_request(method, params, use_cache=False)
 
-    @deprecated_for("is_connected")
-    def isConnected(self) -> bool:
-        return self.is_connected()
-
     def is_connected(self) -> bool:
         provider = self._get_active_provider(use_cache=True)
         return provider is not None and provider.is_connected()
@@ -115,8 +105,7 @@ class AutoProvider(BaseProvider):
         if provider is None:
             raise CannotHandleRequest(
                 "Could not discover provider while making request: "
-                "method:{0}\n"
-                "params:{1}\n".format(method, params)
+                f"method:{method}\nparams:{params}\n"
             )
 
         return provider.make_request(method, params)
