@@ -32,9 +32,11 @@ from .base import (
 
 
 def get_ipc_socket(ipc_path: str, timeout: float = 2.0) -> socket.socket:
-    if sys.platform == 'win32':
+    if sys.platform == "win32":
         # On Windows named pipe is used. Simulate socket with it.
-        from web3._utils.windows import NamedPipe
+        from web3._utils.windows import (
+            NamedPipe,
+        )
 
         return NamedPipe(ipc_path)
     else:
@@ -52,14 +54,19 @@ class PersistantSocket:
 
     def __enter__(self) -> socket.socket:
         if not self.ipc_path:
-            raise FileNotFoundError("cannot connect to IPC socket at path: %r" % self.ipc_path)
+            raise FileNotFoundError(
+                f"cannot connect to IPC socket at path: {self.ipc_path!r}"
+            )
 
         if not self.sock:
             self.sock = self._open()
         return self.sock
 
     def __exit__(
-        self, exc_type: Type[BaseException], exc_value: BaseException, traceback: TracebackType
+        self,
+        exc_type: Type[BaseException],
+        exc_value: BaseException,
+        traceback: TracebackType,
     ) -> None:
         # only close the socket if there was an error
         if exc_value is not None:
@@ -80,127 +87,60 @@ class PersistantSocket:
 
 # type ignored b/c missing return statement is by design here
 def get_default_ipc_path() -> str:  # type: ignore
-    if sys.platform == 'darwin':
-        ipc_path = os.path.expanduser(os.path.join(
-            "~",
-            "Library",
-            "Ethereum",
-            "geth.ipc"
-        ))
-        if os.path.exists(ipc_path):
-            return ipc_path
-
-        ipc_path = os.path.expanduser(os.path.join(
-            "~",
-            "Library",
-            "Application Support",
-            "io.parity.ethereum",
-            "jsonrpc.ipc"
-        ))
-        if os.path.exists(ipc_path):
-            return ipc_path
-
-        base_trinity_path = Path('~').expanduser() / '.local' / 'share' / 'trinity'
-        ipc_path = str(base_trinity_path / 'mainnet' / 'ipcs-eth1' / 'jsonrpc.ipc')
-        if Path(ipc_path).exists():
-            return str(ipc_path)
-
-    elif sys.platform.startswith('linux') or sys.platform.startswith('freebsd'):
-        ipc_path = os.path.expanduser(os.path.join(
-            "~",
-            ".ethereum",
-            "geth.ipc"
-        ))
-        if os.path.exists(ipc_path):
-            return ipc_path
-
-        ipc_path = os.path.expanduser(os.path.join(
-            "~",
-            ".local",
-            "share",
-            "io.parity.ethereum",
-            "jsonrpc.ipc"
-        ))
-        if os.path.exists(ipc_path):
-            return ipc_path
-
-        base_trinity_path = Path('~').expanduser() / '.local' / 'share' / 'trinity'
-        ipc_path = str(base_trinity_path / 'mainnet' / 'ipcs-eth1' / 'jsonrpc.ipc')
-        if Path(ipc_path).exists():
-            return str(ipc_path)
-
-    elif sys.platform == 'win32':
-        ipc_path = os.path.join(
-            "\\\\",
-            ".",
-            "pipe",
-            "geth.ipc"
+    if sys.platform == "darwin":
+        ipc_path = os.path.expanduser(
+            os.path.join("~", "Library", "Ethereum", "geth.ipc")
         )
         if os.path.exists(ipc_path):
             return ipc_path
 
-        ipc_path = os.path.join(
-            "\\\\",
-            ".",
-            "pipe",
-            "jsonrpc.ipc"
-        )
+    elif sys.platform.startswith("linux") or sys.platform.startswith("freebsd"):
+        ipc_path = os.path.expanduser(os.path.join("~", ".ethereum", "geth.ipc"))
+        if os.path.exists(ipc_path):
+            return ipc_path
+
+    elif sys.platform == "win32":
+        ipc_path = os.path.join("\\\\", ".", "pipe", "geth.ipc")
         if os.path.exists(ipc_path):
             return ipc_path
 
     else:
         raise ValueError(
-            "Unsupported platform '{0}'.  Only darwin/linux/win32/freebsd are "
-            "supported.  You must specify the ipc_path".format(sys.platform)
+            f"Unsupported platform '{sys.platform}'.  Only darwin/linux/win32/"
+            "freebsd are supported.  You must specify the ipc_path"
         )
 
 
 # type ignored b/c missing return statement is by design here
 def get_dev_ipc_path() -> str:  # type: ignore
-    if os.environ.get('WEB3_PROVIDER_URI', ''):
-        ipc_path = os.environ.get('WEB3_PROVIDER_URI')
+    if os.environ.get("WEB3_PROVIDER_URI", ""):
+        ipc_path = os.environ.get("WEB3_PROVIDER_URI")
         if os.path.exists(ipc_path):
             return ipc_path
-    elif sys.platform == 'darwin':
-        tmpdir = os.environ.get('TMPDIR', '')
-        ipc_path = os.path.expanduser(os.path.join(
-            tmpdir,
-            "geth.ipc"
-        ))
+    elif sys.platform == "darwin":
+        tmpdir = os.environ.get("TMPDIR", "")
+        ipc_path = os.path.expanduser(os.path.join(tmpdir, "geth.ipc"))
         if os.path.exists(ipc_path):
             return ipc_path
 
-    elif sys.platform.startswith('linux') or sys.platform.startswith('freebsd'):
-        ipc_path = os.path.expanduser(os.path.join(
-            "/tmp",
-            "geth.ipc"
-        ))
+    elif sys.platform.startswith("linux") or sys.platform.startswith("freebsd"):
+        ipc_path = os.path.expanduser(os.path.join("/tmp", "geth.ipc"))
         if os.path.exists(ipc_path):
             return ipc_path
 
-    elif sys.platform == 'win32':
-        ipc_path = os.path.join(
-            "\\\\",
-            ".",
-            "pipe",
-            "geth.ipc"
-        )
+    elif sys.platform == "win32":
+        ipc_path = os.path.join("\\\\", ".", "pipe", "geth.ipc")
         if os.path.exists(ipc_path):
             return ipc_path
 
-        ipc_path = os.path.join(
-            "\\\\",
-            ".",
-            "pipe",
-            "jsonrpc.ipc"
-        )
+        ipc_path = os.path.join("\\\\", ".", "pipe", "jsonrpc.ipc")
         if os.path.exists(ipc_path):
             return ipc_path
 
     else:
         raise ValueError(
-            "Unsupported platform '{0}'.  Only darwin/linux/win32/freebsd are "
-            "supported.  You must specify the ipc_path".format(sys.platform)
+            f"Unsupported platform '{sys.platform}'.  Only darwin/linux/win32/"
+            "freebsd are supported.  You must specify the ipc_path"
         )
 
 
@@ -231,8 +171,9 @@ class IPCProvider(JSONBaseProvider):
         return f"<{self.__class__.__name__} {self.ipc_path}>"
 
     def make_request(self, method: RPCEndpoint, params: Any) -> RPCResponse:
-        self.logger.debug("Making request IPC. Path: %s, Method: %s",
-                          self.ipc_path, method)
+        self.logger.debug(
+            f"Making request IPC. Path: {self.ipc_path}, Method: {method}"
+        )
         request = self.encode_rpc_request(method, params)
 
         with self._lock, self._socket as sock:
