@@ -24,7 +24,10 @@ from web3.tools.benchmark.utils import (
     kill_proc_gracefully,
 )
 
-GETH_FIXTURE_ZIP = "geth-1.10.23-fixture.zip"
+GETH_FIXTURE_ZIP = "geth-1.11.5-fixture.zip"
+
+# use same coinbase value as in `web3.py/tests/integration/generate_fixtures/common.py`
+COINBASE = "0xdc544d1aa88ff8bbd2f2aec754b1f1e99e1812fd"
 
 
 class GethBenchmarkFixture:
@@ -59,7 +62,7 @@ class GethBenchmarkFixture:
         return str(port)
 
     def _endpoint_uri(self) -> str:
-        return "http://localhost:{0}".format(self.rpc_port)
+        return f"http://localhost:{self.rpc_port}"
 
     def _geth_binary(self) -> str:
         if "GETH_BINARY" in os.environ:
@@ -88,6 +91,9 @@ class GethBenchmarkFixture:
             "admin,eth,net,web3,personal,miner",
             "--ipcdisable",
             "--allow-insecure-unlock",
+            "--miner.etherbase",
+            COINBASE[2:],
+            "--rpc.enabledeprecatedpersonal",
         )
 
     def _geth_process(
@@ -101,7 +107,9 @@ class GethBenchmarkFixture:
             str(genesis_file),
         )
         check_output(
-            init_datadir_command, stdin=PIPE, stderr=PIPE,
+            init_datadir_command,
+            stdin=PIPE,
+            stderr=PIPE,
         )
         proc = Popen(
             self._geth_command_arguments(datadir),
